

import java.lang.*;
import java.util.*;
import java.io.*;

public class Raw2D {


  /**
   * One 2D attributed point is:
   * * screen coordinates (x, y)
   * * floatting point attributes (color, normal, z...) for interpolation
   *  
   */
  public static class Attributed2D {

    public int x;
    public int y;
    public double [] attr;

    public Attributed2D(int x_, int y_, double [] attr_){
      x = x_; y = y_; attr = attr_;
    }
  }

  private static double areaOfTriangle(
      Attributed2D a,
      Attributed2D b,
      Attributed2D c) {
    return (double)
        b.x * c.y - b.y * c.x 
      + c.x * a.y - a.x * c.y
      + a.x * b.y - b.x * a.y;
  }
  private static double applyMatrixLine(double area,
      Attributed2D a,
      Attributed2D b,
      int x, int y) {

    return (1.0 / area) * (
        a.x * b.y - b.x * a.y 
        + x * (a.y - b.y)
        + y * (b.x - a.x)
        );

  }

  /**
   * This function fills the triangle delimited by a, b and c, and also 
   * interpolates the attributes.
   *
   * @return a list of Attributed2D: the triangle surface
   */
  public static List fillTriangleWithRectangle(
      Attributed2D a,
      Attributed2D b,
      Attributed2D c) 
  {
    List ret = new LinkedList();

    double area = areaOfTriangle(a, b, c);

    int x_min = Math.min(a.x, Math.min(b.x, c.x));
    int y_min = Math.min(a.y, Math.min(b.y, c.y));
    int x_max = Math.max(a.x, Math.max(b.x, c.x));
    int y_max = Math.max(a.y, Math.max(b.y, c.y));

    for (int x = x_min; x <= x_max; x++) {
      for (int y = y_min; y <= y_max; y++) {
        double phi_1 = applyMatrixLine(area, b, c, x, y);
        if (phi_1 >= 0.) {
          double phi_2 = applyMatrixLine(area, c, a, x, y);
          if (phi_2 >= 0.) {
            double phi_3 = applyMatrixLine(area, a, b, x, y);
            if (phi_3 >= 0.) {
              double attr[] = new double[a.attr.length];
              for (int i = 0; i < a.attr.length; i++) {
                attr[i] = 
                  phi_1 * a.attr[i] + phi_2 * b.attr[i] + phi_3 * c.attr[i];
              }
              ret.add(new Attributed2D(x, y, attr)) ;
            }
          }
        }
      }
    }
    ret.add(a);
    ret.add(b);
    ret.add(c);

    return ret ;
  }


  private static double [] interp2Attrs(
      int x , int y,
      int x0, int y0, double []attr0,
      int x1, int y1, double []attr1)
  {
    double [] ret = new double[attr0.length];

    double coef ;
    if ( Math.abs(x1 - x0) > Math.abs(y1 - y0) ) {
      coef = (double)(x - x0) / (double)(x1 - x0) ;
    } else {
    	if (y1 != y0) {
    		coef = (double)(y - y0) / (double)(y1 - y0) ;
    	} else {
    		coef = 0.5 ;
    	}
    }
    for (int i = 0 ; i < attr0.length ; i++) {
      ret[i] = (1.0 - coef) * attr0[i] + ( coef) * attr1[i] ;
    }
    return ret;
  }


  /**
   * This function builds the line between points 'a' and 'b',
   * it also interpolates the attributes of the points along the line.
   *
   * @return a list of Attributed2D ready to draw.
   */
  public static List bresenhamAndInterpol(
      Attributed2D a, Attributed2D b)
  {

    List ret = new LinkedList();


    int x0 = a.x;
    int x1 = b.x;
    int y0 = a.y;
    int y1 = b.y;
    double [] attr0 = a.attr ;
    double [] attr1 = b.attr ;

    boolean sym = Math.abs(y1 - y0) > Math.abs(x1 - x0);
    if (sym) {
      int aux;
      aux = x0; x0 = y0 ; y0 = aux; // swap
      aux = x1; x1 = y1 ; y1 = aux; // swap
    }
    if (x0 > x1) {
      double [] attrAux ;
      int aux;
      aux = x0; x0 = x1 ; x1 = aux; // swap
      aux = y0; y0 = y1 ; y1 = aux; // swap
      attrAux = attr0 ; attr0 = attr1 ; attr1 = attrAux ;
    }
    int ystep;
    if (y0 < y1) {
      ystep =  1;
    } else {
      ystep = -1;
    }

    int err = (x0 - x1) / 2;
    int dx = (x1 - x0); 
    int dy = Math.abs(y1 - y0);

    int x = x0;
    int y = y0;

    while (x <= x1) {
      double [] attr = interp2Attrs(x,y,x0,y0,attr0,x1,y1,attr1);
      if (sym)
        ret.add(new Attributed2D(y,x, attr));
      else
        ret.add(new Attributed2D(x,y, attr));
      x += 1;
      err = err + dy;
      if (err > 0) {
        y += ystep;
        err -= dx;
      }
    }

    return ret ;
  }




}
